/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distribselectGameuted on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Wikipedia Toolbar.
 *
 * The Initial Developer of the Original Code is
 * Bananeweizen
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Shane A. Froebel <shane.froebel@gmail.com/bugs@bugssite.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 /* ***** CVS HEADERS *****
 * $Id: functions.js,v 1.13 2006/05/16 20:31:03 froebel Exp $
 * ***** END CVS HEADERS ***** */

const IMAGE_ON = "chrome://wikipedia/skin/images/select_on.gif";
const IMAGE_OFF = "chrome://wikipedia/skin/images/select_off.gif";

const OPTION_ACTIVE = "active";
const OPTION_USER_FORMAT = "userformat";
const OPTION_USER_TEMPLATE = "usertemplate";
const OPTION_USER_TEMPLATE_CAT = "usertemplatecat";
const OPTION_USER_WIKI = "userwiki";	//	full system
const OPTION_USER_WIKI_SITE = "userwiki.site";	//	full system
const OPTION_USER_WIKI_TEMPLATES = "userwiki.tmplonly";	//	templates only
const OPTION_USER_WIKI_NOINSTALL = "userwiki.noinstall";	//	drop down only	
const OPTION_SHORTCUT = "shortcut";
const OPTION_HISTORY = "history";

var wpPrefBranch = null;

/*********** MAIN AREA ***************/

function trim(input) {
	var s = input;
	try {
		if (s == null || s == "") {
			return "";
		}
		while ((s.substring(0,1) == ' ') || (s.substring(0,1) == '\n')) {
			s = s.substring(1,s.length);
		}
		while ((s.substring(s.length-1,s.length) == ' ') || (s.substring(s.length-1,s.length) == '\n')) {
			s = s.substring(0,s.length-1);
		}
	}
	catch (e) {
	}
	return s;
}

function replace(input, search, replace) {
	var output = "" + input;
	while (output.indexOf(search)>-1) {
		var pos= output.indexOf(search);
		output = "" + (output.substring(0, pos) + replace + output.substring((pos + search.length), output.length));
	}
	return output;
}

function blankLeft(s) {
	if (s == null || s == "") {
		return "";
	}
	var r = "";
	while (s.substring(0,1) == ' ') {
		r = r+s.substring(0,1);
		s = s.substring(1,s.length);
	}
	return r;
}

function blankRight(s) {
	if (s == null || s == "") {
		return "";
	}
	var r = "";
	while (s.substring(s.length-1,s.length) == ' ') {
		r = s.substring(s.length-1,s.length);
		s = s.substring(0,s.length-1);
	}
	return r;
}

function formatString(aString, arguments) {
	var result = aString;
	for (var i = 0; i < arguments.length; i++) {
		var regEx = new RegExp ('%'+(i+1), 'gi') ;
		result = result.replace(regEx, arguments[i]);
	}
	return result;
}

function readFile(str_Filename)
{
	try{
		var obj_File = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		obj_File.initWithPath(str_Filename);

		var obj_InputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
		obj_InputStream.init(obj_File,0x01,0444,null);

		var obj_ScriptableIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
		obj_ScriptableIO.init(obj_InputStream);
	}
	catch (e) {
		alert(e);
	}

	try {
		var str = obj_ScriptableIO.read(obj_File.fileSize-1);
	}
	catch (e) {
		dump(e);
	}

	obj_ScriptableIO.close();
	obj_InputStream.close();
	return str;
}

function getContentsFromURL(aURL){
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"].getService(Components.interfaces.nsIScriptableInputStream);
	var channel = ioService.newChannel(aURL,null,null);
	var input = channel.open();
	scriptableStream.init(input);
	var str = scriptableStream.read(input.available());
	scriptableStream.close();
	input.close();
	return str;
}

function getNodes(url, nodeSelector) {
	var content = getContentsFromURL(url);
	var uConv = Components.classes['@mozilla.org/intl/scriptableunicodeconverter'].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
	uConv.charset = "UTF-8";
	content = uConv.ConvertToUnicode(content);
	var parser = new DOMParser();
	var domtree = parser.parseFromString(content, "text/xml");
	var nodes = document.evaluate(nodeSelector,domtree,null,0,null);
	var nodelist = new Array();
	var xpathnode = nodes.iterateNext();
	while (xpathnode) {
		nodelist.push(xpathnode);
		xpathnode = nodes.iterateNext();
	}
	return nodelist;
}

/*********** PREFERENCES AREA ***************/

function getPreferences() {
	if (!wpPrefBranch) {
		try {
			var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			wpPrefBranch = prefService.getBranch("wikipedia."); // preferences wikipedia node
		}
		catch (e) {
			alert(e);
		}
	}
	return wpPrefBranch;
}

function onPrefsLoad() {
	//	MediaWiki
	onLoadPageUser();
	//	User Wikis
	onLoadPageWikis();
	//	Edit Prefs
	onLoadPageEdit();
	//	View Prefs
	onLoadPageView();
	//	Format Prefs
	onLoadPageFormats();
	//	Templates Prefs
	onLoadPageTemplates();
}

function onPrefsSave() {
	//	Edit Prefs
	onSavePageEdit();
	//	Edit Formats
	onSavePageFormats();
	
	document.getElementById("wikipedia.apply").disabled = true;
}

function onPrefsOnAction(aEvent) {
    if ((aEvent.originalTarget.localName == "button") ||
        (aEvent.originalTarget.localName == "tab") ||
        (aEvent.originalTarget.getAttribute("type") == "noapply")) return;
    
    // Enable Apply button
    document.getElementById("wikipedia.apply").disabled = false;
}