/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distribselectGameuted on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Wikipedia Toolbar.
 *
 * The Initial Developer of the Original Code is
 * Bananeweizen
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Shane A. Froebel <shane.froebel@gmail.com/bugs@bugssite.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 /* ***** CVS HEADERS *****
 * $Id: wikipedia.js,v 1.27 2006/05/17 12:38:26 froebel Exp $
 * ***** END CVS HEADERS ***** */
 
var wpPrefBranch = null;
var wpStringBundle;
var wpDocument = null;
var currentBox;
var scrollTop;
var scrollHeight;
var wpSiteValue;
var wpSelectionStart = -1;
var wpSelectionEnd = -1;
var wpWikiUrl = "";
var wpWikiLanguage = "";
var wpWikiType = "";
var wpCheckBoxFormats = null;
var wpSelectEventEnabled = true;
var wpIsMozilla = false;
var wpIsFirefox = true;
var wpEnableButtonIds = ['link','bold','italic','format-menu','paragraph-menu','list-menu','template-menu','history','table','image','signature','weblink','specialcharacter-menu','horline'];
var wpCheckBoxIds = ['link','bold','italic','underline','strike','pre','code','tt','small','sup','sub','nowiki'];
var wpSupportedFormats = [];
var wpAllFormats = ['link','pipedlink','bold','italic','underline','strike','small','code','pre','tt','sup','sub','nowiki','indentation','bulletlist','numberedlist','definitionlist','signature','headline2','headline3','headline4','weblink','pipedweblink','horline'];
var wpInited = false;
var wpCurrentUrl = "";
var wpIsEditPage = false;

var consoleService = Components.classes['@mozilla.org/consoleservice;1']
                               .getService(Components.interfaces.nsIConsoleService);

var wikipedia = {
	init: function() {
		//initialize only once
		if (wpInited == false) {
			wpDocument = document;
			wpStringBundle = wpDocument.getElementById("wikipediastrings");
			wpIsMozilla = !document.getElementById("customToolbars");
			if (wpIsMozilla) {
				wpIsFirefox = false;
			} else {
				wpIsFirefox = true;
			}
			
			wpCurrentUrl = window._content.location.href;

			//fix toolbar layout   TODO: change on setting navigator preferences (Mozilla only)
			wikipedia.fixToolbarLayout();

			// setup according to extension preferences
			document.getElementById('wikipedia-specialpages-restricted').setAttribute("hidden",!wikipedia.getPreferences().getBoolPref("restrictedspecial"));
			wikipedia.setCurrentProject();

			//add event listeners
			var menu = document.getElementById('contentAreaContextMenu');
			menu.addEventListener('popupshowing', wikipedia.popupShowingEvent, false);
			addEventListener("focus",wikipedia.focusEvent,false);
			addEventListener("select",wikipedia.selectEvent,false);
			addEventListener("keyup",wikipedia.keyupEvent,false);
			addEventListener("click",wikipedia.clickEvent,false);

			//add preferences observer
			var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal);
			prefService.addObserver("wikipedia",wikipedia,false);

			// add progress listener to capture location changes
			window.getBrowser().addProgressListener(wikipedia, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT | Components.interfaces.nsIWebProgress.NOTIFY_STATE_WINDOW );

			//	reset the history count, we are back at zero!
			wikipedia.getPreferences().setIntPref(OPTION_HISTORY+".count", 0);

			//eventually hide toolbar
			wikipedia.showHideToolbar();

			wikipedia.setShortcuts();

			wpInited = true;
		}
		return true;
	},

	fixToolbarLayout: function() {
		if (wpIsMozilla) {
			wpDocument.getElementById('wikipedia-toolbar').setAttribute("buttonstyle",document.getElementById('nav-bar').getAttribute("buttonstyle"));
			for (var i = 0; i < wpEnableButtonIds.length; i++) {
				var button = document.getElementById('wikipedia-'+wpEnableButtonIds[i]);
				if (button != null) {
					button.setAttribute("buttonstyle",document.getElementById('nav-bar').getAttribute("buttonstyle"));
				}
			}
			document.getElementById('wikipedia-menu').setAttribute("buttonstyle",document.getElementById('nav-bar').getAttribute("buttonstyle"));
		} else {
			var iconsize = wikipedia.getPreferences().getIntPref("icon.size");
			if (iconsize == 1) {
				document.getElementById('wikipedia-toolbar').setAttribute('iconsize', 'small');
			} else {
				document.getElementById('wikipedia-toolbar').setAttribute('iconsize', '');
			}
			document.getElementById('wikipedia-toolbar').setAttribute("mode",document.getElementById('navigator-toolbox').getAttribute("mode"));
		}
	},

	setShortcuts: function() {
		var wpKeys = Array('link','bold','italic','table','image','signature','weblink');
		for (var i = 0; i < wpKeys.length; i++) {
			var modifiers = "";
			var key = "";
			var keycode = "";
			try {
				modifiers = wikipedia.getPreferences().getCharPref("shortcut."+wpKeys[i]+".modifiers");
				key = wikipedia.getPreferences().getCharPref("shortcut."+wpKeys[i]+".key");
				keycode = wikipedia.getPreferences().getCharPref("shortcut."+wpKeys[i]+".keycode");
			}
			catch (e) {
			}
			var keyNode = document.getElementById('key_'+wpKeys[i]);
			if (modifiers != "") {
				keyNode.setAttribute('modifiers',modifiers);
			}
			if (key != "") {
				keyNode.setAttribute('key',key);
			}
			if (keycode != "") {
				keyNode.setAttribute('keycode',keycode);
			}
		}
	},

	showHideToolbar: function() {
		var kind = 0;
		var activate = true;
		var condition = "wiki";
		try {
			kind = wikipedia.getPreferences().getIntPref("activation.kind");
			condition = wikipedia.getPreferences().getCharPref("activation.condition");
			if (kind == 1) {
				activate = (wpCurrentUrl.indexOf(condition) >= 0);
			}
		}
		catch (e) {
		}
		document.getElementById('wikipedia-toolbar').hidden = !activate;
		return activate;
	},

	popupShowingEvent: function() {
		try {
			var activate = wikipedia.showHideToolbar();
			// edit items visibility
			var hideEdit = true;
			if (document.popupNode != null) {
				hideEdit = (document.popupNode.tagName!='TEXTAREA') || !activate;
				if (!hideEdit) {
					hideEdit = document.popupNode.getAttribute("readonly") == "readonly";
				}
			}
			var ids = ['specialcharacters','separator-link','link','bold','italic','format','separator-paragraph','paragraph','list','template','usertemplate','table','image','signature','horline','weblink','history'];
			for (var i = 0; i < ids.length; i++) {
				document.getElementById('context-wikipedia-' + ids[i]).hidden = hideEdit;
			}
			// search menu item label and visibility
			var hideSearch = false;
			if (gContextMenu.isTextSelected) {
				//	gContextMenu.isTextSelected
				var search = trim(gContextMenu.searchSelected());
				if (search == "") {
					try {
						search = trim(wikipedia.getSelection());
					}
					catch (e) {
					}
				}
				if (search.length > 20) {
					search = search.substring(0,20)+"...";
				}
	            var searchLabel = formatString(wikipedia.getString("search.label"),new Array(search));
				document.getElementById('context-wikipedia-search').setAttribute("label", searchLabel);
				hideSearch = ((search == "") || (!activate));
				document.getElementById('context-wikipedia-search').hidden = hideSearch;
			} else {
				document.getElementById('context-wikipedia-search').hidden = !hideSearch;
			}
			// page related links
			var hidePage = (wikipedia.getArticleName() == "") || !activate || (wpIsEditPage && !(document.popupNode.tagName == 'A'));
			document.getElementById('context-wikipedia-page-edit').hidden = hidePage;
			document.getElementById('context-wikipedia-page-versions').hidden = hidePage;
			document.getElementById('context-wikipedia-page-watch').hidden = hidePage;
			document.getElementById('context-wikipedia-page-unwatch').hidden = hidePage;
			document.getElementById('context-wikipedia-page-move').hidden = hidePage;
			document.getElementById('context-wikipedia-page-whatlinkshere').hidden = hidePage;
			document.getElementById('context-wikipedia-page-recentchangeslinked').hidden = hidePage;
			document.getElementById('context-wikipedia-separator-search').hidden = hideEdit || (hideSearch && hidePage);
			document.getElementById('context-wikipedia-page-addwiki').hidden = hidePage;
			// other items
			document.getElementById('context-wikipedia-articleinfo').hidden = hideEdit;
			hideCheckBoxes = !wikipedia.checkBoxesAvailable();
			document.getElementById('context-wikipedia-checkall').hidden = hideCheckBoxes;
			document.getElementById('context-wikipedia-separator-addwiki').hidden = hidePage || (hideEdit && hideCheckBoxes);
			// visibility of complete menu
			var menuitems = document.getElementById('context-wikipedia').firstChild.childNodes;
			var countVisible = 0;
			for (var i = 0; i < menuitems.length; i++) {
				var item = menuitems[i];
				if (item.getAttribute('hidden') != 'true') {
					countVisible++;
				}
			}
			document.getElementById('context-wikipedia').hidden = (countVisible == 0);
		}
		catch(e) {
			dump(e);
		}
	},

	focusEvent: function() {
		try {
			wikipedia.enableButtons();
		}
		catch (e) {
		}
		return true;
	},

	enableButtons: function() {
		var enable = wikipedia.isWritableEditFieldSelected();
		var isEnabled = !wpDocument.getElementById('wikipedia-'+wpEnableButtonIds[0]).disabled;
		if (isEnabled != enable) {
			for (var i = 0; i < wpEnableButtonIds.length; i++) {
				var button = wpDocument.getElementById('wikipedia-'+wpEnableButtonIds[i]);
				if (button != null) {
					button.disabled = !enable;
				}
			}
		}
	},

	uncheckCheckboxes: function() {
		for (var i = 0; i < wpCheckBoxIds.length; i++) {
			button = document.getElementById('wikipedia-'+wpCheckBoxIds[i]);
			if (button != null) {
				button.checked=false;
			}
			document.getElementById('context-wikipedia-'+wpCheckBoxIds[i]).checked = false;
		}
	},

	stringHasFormat: function(selection, format) {
		var begin = format.substring(0,format.indexOf("%"));
		var end = format.substring(format.indexOf("1")+1,format.length);
		return (selection.length >= begin.length+end.length+1) && (selection.substring(0,begin.length) == begin) && (selection.substring(selection.length-end.length,selection.length) == end);
	},

	getSelectionFormat: function(extended) {
		var selection = wikipedia.getSelection();
		var allText = currentBox.value;
		// read all checkbox formats into a cache, otherwise we reread the XML for every keystroke
		if (wpCheckBoxFormats == null) {
			wpCheckBoxFormats = new Array();
			for (var i = 0; i < wpCheckBoxIds.length; i++) {
				var format = wikipedia.getFormatAttribute(wpCheckBoxIds[i],"formatstring");
				wpCheckBoxFormats.push(format);
			}
		}
		for (var i = 0; i < wpCheckBoxIds.length; i++) {
			format = wpCheckBoxFormats[i];
			var begin = format.substring(0,format.indexOf("%"));
			var end = format.substring(format.indexOf("1")+1,format.length);
			var found = false;
			if (!extended) {
				found = (selection.length >= begin.length+end.length+1) && (selection.substring(0,begin.length) == begin) && (selection.substring(selection.length-end.length,selection.length) == end);
			}
			else {
				found = (allText.substring(wpSelectionStart-begin.length,wpSelectionStart) == begin) && (allText.substring(wpSelectionEnd,wpSelectionEnd+end.length) == end);
			}
			if (found) {
				return i;
			}
		}
		return -1;
	},

	selectEvent: function() {
		try {
			if (wpSelectEventEnabled) {
				if (!wikipedia.isEditFieldSelected()) {
					wikipedia.uncheckCheckboxes();
				} else {
					var formatIndex = wikipedia.getSelectionFormat(false);
					if (formatIndex < 0) {
						formatIndex = wikipedia.getSelectionFormat(true);
					}
					for (var i = 0; i < wpCheckBoxIds.length; i++) {
						var checked = (i == formatIndex);
						var button = document.getElementById('wikipedia-'+wpCheckBoxIds[i]);
						if (button != null) {
							button.checked = checked;
						}
						document.getElementById('context-wikipedia-'+wpCheckBoxIds[i]).checked = checked;
					}
				}
			}
		}
		catch (e) {
		}
	},

	keyupEvent: function() {
		try {
			wikipedia.selectEvent();
		}
		catch (e) {
		}
	},

	clickEvent: function() {
		try {
			wikipedia.enableButtons();
			wikipedia.selectEvent();
		}
		catch (e) {
		}
	},

	getClipboard: function() {
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
		if (!clip){
			return "";
		}
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
		if (!trans){
			return "";
		}
		trans.addDataFlavor("text/unicode");
		clip.getData(trans,clip.kGlobalClipboard);
		var str_clipboard = new Object();
		var strLength = new Object();
		try {
			trans.getTransferData("text/unicode",str_clipboard,strLength);
			if (str_clipboard){
				str_clipboard = str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
			}
		}
		catch (e){
			//alert("No text in the clipboard, please copy something first.");
		}
		return str_clipboard;
	},

	getSelection: function() {
		if (wikipedia.isEditFieldSelected()) {
			currentBox = document.commandDispatcher.focusedElement;
			scrollTop = currentBox.scrollTop;
			scrollHeight = currentBox.scrollHeight;
			//Get Selected text (if selected)
			wpSelectionStart = currentBox.selectionStart;
			wpSelectionEnd = currentBox.selectionEnd;
			return currentBox.value.substring(wpSelectionStart, wpSelectionEnd);
		}
		return "";
	},

	getSelectionAll: function() {
		currentBox.setSelectionRange(0,currentBox.value.length-1);
		return wikipedia.getSelection();
	},

	getForcedSelection: function(format) {
		var str = wikipedia.getSelection();
		if (str==null || str=="") {
			str=wikipedia.getInput("input."+format);
		}
		return str;
	},

	insert: function(someText) {
		wikipedia.getSelection();
		wikipedia.insertAtCursor(someText);
	},

	surround: function(format, left,right) {
		var str = wikipedia.getSelection();
		if (format != "") {
			str = wikipedia.getForcedSelection(format);
		}
		var leftTrim = "";
		while ((str.length > 0) && (str.substring(0,1) == ' ')) {
			leftTrim = leftTrim+" ";
			str = str.substring(1,str.length);
		}
		var rightTrim = "";
		while ((str.length > 0) && (str.substring(str.length-1,str.length) == ' ')) {
			rightTrim = rightTrim+" ";
			str = str.substring(0,str.length-1);
		}
		wikipedia.insertAtCursor(leftTrim+left+str+right+rightTrim);
	},

	applyFormat: function(formatName) {
		var selection = wikipedia.getForcedSelection(formatName);
		if (selection != "") {
			var autotrim = false;
			try {
				autotrim = wikipedia.getFormatAttribute(formatName,"autotrim");
			} catch (e) {
			}
			var left = "";
			var right = "";
			if (autotrim) {
				left = blankLeft(selection);
				right = blankRight(selection);
				selection = trim(selection);
			}
			selection = formatString(wikipedia.getFormatAttribute(formatName,"formatstring"),new Array(selection));
			wikipedia.insertAtCursor(left+selection+right);
		}
		wikipedia.selectEvent();
	},

	applyFormatString: function(userFormat) {
		var selection = wikipedia.getSelection();
		if (selection != "") {
			var autotrim = true;
			if (autotrim) {
				left = blankLeft(selection);
				right = blankRight(selection);
				selection = trim(selection);
			} else {
				left = "";
				right = "";
			}
			selection = formatString(userFormat,new Array(selection));
			wikipedia.insertAtCursor(left+selection+right);
		}
		wikipedia.selectEvent();
	},

	removeFormat: function(formatName) {
		var selection = wikipedia.getSelection();
		if (selection != "") {
			var format = wikipedia.getFormatAttribute(formatName,"formatstring");
			wikipedia.removeFormatString(format,formatName);
		}
	},

	removeFormatString: function(format, formatName) {
		var selection = wikipedia.getSelection();
		if (selection != "") {
			var begin = format.substring(0,format.indexOf("%"));
			var end = format.substring(format.indexOf("1")+1,format.length);
			var formatIndex = wikipedia.getSelectionFormat(false);
			if ((formatIndex >= 0) || (formatName.indexOf('userformat') == 0)) {
				// standard selection
				selection = selection.substring(begin.length,selection.length-end.length);
			} else {
				// extended selection, correct it
				wpSelectEventEnabled = false;
				currentBox.setSelectionRange(wpSelectionStart-begin.length,wpSelectionEnd+end.length);
				wpSelectionStart = wpSelectionStart-begin.length;
				wpSelectEventEnabled = true;
			}
			wikipedia.insertAtCursor(selection);
			wikipedia.selectEvent();
		}
	},

	toggleFormat: function(formatName) {
		var state = document.getElementById('context-wikipedia-'+formatName).checked;
		if (state == false) {
			wikipedia.applyFormat(formatName);
			this.addHistory("Apply Format: " + formatName, "wikipedia.applyFormat('"+formatName+"')");
		} else {
			wikipedia.removeFormat(formatName);
			this.addHistory("Remove Format: " + formatName, "wikipedia.removeFormat('"+formatName+"')");
		}
	},

	getFormatAttribute: function(formatName, attribute) {
		var nodeList = getNodes("chrome://wikipedia/content/wikisoftware/"+wpWikiType+".xml","/wikisoftware/formats/format[attribute::id='"+formatName+"']");
		if (nodeList.length > 0) {
			node = nodeList[0];
			return node.getAttribute(attribute);
		}
		return "";
	},

	applyFormats: function(formatName, arguments) {
		var nodeList = getNodes("chrome://wikipedia/content/wikisoftware/"+wpWikiType+".xml","/wikisoftware/formats/format[attribute::id='"+formatName+"']");
		var node = nodeList[0];
		var format = node.getAttribute("formatstring");
		for (var i = 0; i < arguments.length; i++) {
			var regEx = new RegExp ('%'+(i+1), 'gi') ;
			format = format.replace(regEx, arguments[i]);
		}
		return format;
	},

	insertFormat: function(formatName) {
		wikipedia.insert(wikipedia.getFormatAttribute(formatName,"formatstring"));
	},

	changeParagraphsInLine: function(line, heading, nextHeading) {
		var pos = 0;
		while (pos < line.length) {
			if ((line.substring(pos,pos+heading.length) == heading) && (line.substring(pos,pos+nextHeading.length) != nextHeading)) {
				line = line.substring(0,pos)+nextHeading+line.substring(pos+heading.length);
				pos = pos+heading.length;
			} else {
				pos++;
			}
		}
		return line;
	},

	changeParagraphs: function(change) {
		var str = wikipedia.getSelection();
		if (str == "") {
			str = wikipedia.getSelectionAll();
		}
		var lines = str.split("\n");
		var insert = "";
		for (var i = 0; i < lines.length; i++) {
			var line = lines[i];
			if (line.indexOf("==") >= 0) {
				if (change == 1) {
					line = wikipedia.changeParagraphsInLine(line, '====', '=====');
					line = wikipedia.changeParagraphsInLine(line, '===', '====');
					line = wikipedia.changeParagraphsInLine(line, '==', '===');
				} else {
					line = line.replace(new RegExp ('===', 'gi'), '==') ;
					line = line.replace(new RegExp ('====', 'gi'), '===') ;
					line = line.replace(new RegExp ('=====', 'gi'), '====') ;
				}
			}
			if (i > 0) {
				insert = insert+"\n";
			}
			insert = insert+line;
		}
		wikipedia.insertAtCursor(insert);
	},

	applyHeadline: function(formatName) {
		var sel = wikipedia.getSelection();
		if (sel == "") {
			wikipedia.applyFormat(formatName);
		} else {
			sel = trim(sel);
			while (sel.indexOf('=') == 0) {
				sel = sel.substring(1);
			}
			while (sel.lastIndexOf('=') == sel.length-1) {
				sel = sel.substring(0,sel.length-1);
			}
			sel = trim(sel);
			wikipedia.insert(wikipedia.applyFormats(formatName,new Array(sel)));
		}
		this.addHistory("Apply Heading: " + formatName, "wikipedia.applyHeadline('"+formatName+"')");
	},

	insertSignature: function() {
		var trailing = "";
		wikipedia.getSelection();
		var selStartPos = currentBox.selectionStart;
		if (selStartPos > 0) {
			var before = currentBox.value.substring(selStartPos-1, selStartPos);
			if ((before != ' ') && (before != '\n')) {
				trailing = " ";
			}
		}
		var signature="";
		if (wikipedia.getPreferences().getIntPref("signature.selection") == 0) {
			signature = wikipedia.getFormatAttribute("signature","formatstring");
		} else {
			signature = wikipedia.getPreferences().getCharPref("signature.text");
		}
		wikipedia.insert(trailing+signature);
	},

	insertAtCursor: function(aText) {
		//replace the "\n" in our wikisoftware formats by a real \n
		aText = replace(aText, "\\n", "\n");
		try {
			var command = "cmd_insertText";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				var selectionStartBefore=wpSelectionStart;
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				params.setStringValue("state_data", aText);
				controller.doCommandWithParams(command, params);
				//restore position
				var nHeight = currentBox.scrollHeight - scrollHeight;
				currentBox.scrollTop = scrollTop + nHeight;
				//restore selection
				currentBox.setSelectionRange(selectionStartBefore,selectionStartBefore+aText.length);
			}
		}
		catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},

	insertList: function(listName) {
		var listSign = wikipedia.getFormatAttribute(listName,"formatstring");
		var str= wikipedia.getSelection();
		var lines = str.split("\n");
		var insert = "";
		for (var i = 0; i < lines.length; i++) {
			if (i > 0) {
				insert = insert+"\n";
			}
			if (trim(lines[i]) != "") {
				insert = insert+listSign+lines[i];
			}
		}
		if (str == "") {
			for (i = lines.length; i < 3; i++) {
				insert = insert+"\n"+listSign;
			}
		}
	  	wikipedia.insertAtCursor(insert);
	},

	insertDefinition: function() {
		var str = wikipedia.getSelection();
        var ok = [false];
		str = trim(str);
        var params = [str];
        window.openDialog("chrome://wikipedia/content/wikipedia_definition.xul", "Definition","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]){
        	var topic = trim(params[0]);
        	var explain = trim(params[1]);
        	var definition = wikipedia.applyFormats("definitionlist",new Array(topic,explain));
			wikipedia.insertAtCursor(definition);
		}
	},

	removeList: function() {
		var lines = wikipedia.getSelection().split("\n");
		for (var i = 0; i < lines.length; i++) {
			if (lines[i].length > 0) {
				var c = lines[i].substring(0,1);
				if ((c == '*') || (c == '#') || (c == ':') || (c == ' ')) {
					lines[i] = trim(lines[i].substr(1,lines[i].length-1));
				}
			}
		}
		var insert = lines[0];
		for (var i = 1; i < lines.length; i++) {
			insert = insert+"\n"+lines[i];
		}
	  	wikipedia.insertAtCursor(insert);
	},

	insertTable: function() {
		var str = wikipedia.getSelection();
        var ok = [false];
        var params = [0,0,trim(str),false,false,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/wikipedia_table.xul", "WikiTable","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]) {
        	var rows = Math.round(params[0]);
        	var columns = Math.round(params[1]);
        	var content = params[2];
        	var border = params[3];
        	var headline = params[4];
			var caption = trim(params[5]);
			var alignment = params[6];
			var color = params[7];
        	var headings = params[8];
        	var widths = params[9];

        	var table="{|";
        	if (border) {
        		table = table+" border=\"1\"";
        	}
        	switch (alignment) {
        		case 1:
        			table = table+" align=\"left\"";
        			break;
        		case 2:
        			table = table+" align=\"center\"";
        			break;
        		case 3:
        			table = table+" align=\"right\"";
        			break;
        	}
        	table = table+"\n";
        	if (caption != "") {
        		table = table+"|+"+caption+"\n";
        	}
        	// column headings
       		var cellStart = "|";
       		if (headline) {
       			cellStart = "!";
       		}
    		for (var c = 0; c < columns; c++) {
    			var cellWidth = "";
    			var cellContent = "";
    			if (c < 10) {
    				cellContent = headings[c];
    				if (widths[c] != "") {
	    				cellWidth = "width="+widths[c]+"|";
	    			} else {
	    				cellWidth = "";
	    			}
    			}
    			table = table+cellStart+cellWidth+cellContent+"\n";
    		}
       		table = table+"|-\n";
    		// content
        	for (var r = 0; r < rows; r++) {
        		for (var c = 0; c < columns; c++) {
        			if ((c < 10) && (r < 10)) {
        				cellContent=content[r][c];
        			} else {
        				cellContent = "";
        			}
        			table = table+"|"+cellContent+"\n";
        		}
        		if (r < rows-1) {
	        		table = table+"|-";
					if (color != "") {
		        		table = table+" bgcolor="+color;
        			}
	        		table = table+"\n";
	        	}
        	}
        	table = table+"|}\n";
			wikipedia.insertAtCursor(table);
        }
	},

	insertImage: function() {
		var str = trim(wikipedia.getSelection());
        var ok = [false];
        // use selected string as either image name or description
        var imageName = "";
        var imageDesc = "";
        if (str.indexOf(".") >= 0) {
        	imageName = str;
        } else {
        	imageDesc = str;
        }
        // call dialog
        var params = [imageName,imageDesc,true,false,false,false,0,false,false,false,0,0,0,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/wikipedia_image.xul", "WikiImage","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]) {
        	var file = trim(params[0]);
        	var description = trim(params[1]);
        	var thumb = params[2];
        	var thumbsized = params[3];
        	var framed = params[4];
        	var full = params[5];
        	var pixels = params[6];
        	var alignment = params[7];
        	var imageNamespace = wikipedia.getLanguageString("image");
        	if (imageNamespace == "") {
        		imageNamespace = "Image";
        	}
        	var image = "[["+imageNamespace+":"+file;
        	if (framed) {
        		image = image+"|framed";
        	}
        	else if (thumb) {
        		image = image+"|thumb";
        	}
        	else if (thumbsized) {
        		image = image+"|thumb|"+pixels+"px";
        	}
        	switch (alignment) {
	        	case 1:
	        		image = image+"|left";
	        		break;
	        	case 2:
	        		image = image+"|none";
	        		break;
	        	case 3:
	        		image = image+"|right";
	        		break;
				default:
	        		image = image+"|none";
        	}
        	if (description != "") {
	        	image = image+"|"+description;
	        }
        	image = image+"]]";
        	if (alignment == 2) {
        		image = "<div align=\"center\">"+image+"</div>";
        	}
			wikipedia.insertAtCursor(image);
			this.addHistory("Insert Image: " + file, "wikipedia.insertAtCursor('"+image+"')");
        }
	},

	insertLink: function() {
		var str = wikipedia.getSelection();
        var ok = [false];
        var left = blankLeft(str);
        var right = blankRight(str);
		str = trim(str);
		var str_target = "";
		var str_visible = "";
		var showDialog = wikipedia.getPreferences().getBoolPref("linkdialog")
		if ((!showDialog) || (wikipedia.getPreferences().getIntPref("selectionlink") == 0)) {
			str_target = str;
		}
		else {
			str_visible = str;
		}
        var params = [str_target,str_visible,"",true,0,0,0,0];
        if (showDialog) {
	        window.openDialog("chrome://wikipedia/content/wikipedia_link.xul", "WikiLink","chrome,modal,centerscreen,resizable",ok,params);
	    }
        if ((ok[0]) || (!showDialog)){
        	var target = trim(params[0]);
        	var label = trim(params[1]);
        	var interwiki = trim(params[2]);
        	var link = target;
        	if (interwiki != "") {
        		link = ":"+interwiki+":"+link;
        	}
        	if (label == "") {
        		link = wikipedia.applyFormats("link",new Array(link));
        	}
        	else {
        		link = wikipedia.applyFormats("pipedlink",new Array(link,label));
        	}
			wikipedia.insertAtCursor(left+link+right);
        }
	},

	toggleLink: function() {
		var state = document.getElementById('wikipedia-link').checked;
		if (state == false) {
			wikipedia.insertLink();
		} else {
			wikipedia.removeFormat('link');
		}
		wikipedia.selectEvent();
	},

	insertWebLink: function() {
		if (document.getElementById("wikipedia-weblink").disabled) {
			return;
		}
		var str = wikipedia.getSelection();
        var ok = [false];
        var left = blankLeft(str);
        var right = blankRight(str);
		str = trim(str);
		var str_target = "http://";
		var str_visible = "";
		if ((str.indexOf("http") >= 0) || (str.indexOf("www.") >= 0)) {
			str_target = str;
		} else {
			str_visible = str;
		}
        var params = [str_target,str_visible,"",false,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/wikipedia_link.xul", "WikiLink","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]) {
        	var target = trim(params[0]);
        	if (target.indexOf("://") < 0) {
        		target = "http://"+target;
        	}
        	var label = trim(params[1]);
        	var link = replace(target," ","?space?");
        	if (label == "") {
        		link = wikipedia.applyFormats("weblink",new Array(link));
        	} else {
        		link = wikipedia.applyFormats("pipedweblink",new Array(link,label));
        	}
        	link = replace(link,"?space?","%20");
			wikipedia.insertAtCursor(left+link+right);
		}
	},

	getUsername: function() {
		var username = "";
		try {
			username = wikipedia.getPreferences().getCharPref("username");
		}
		catch(e) {
		}
		if (username == "") {
			alert(wikipedia.getString("settings.username.undefined"));
			wikipedia.toolbarOptions();
		}
		return wikipedia.getPreferences().getCharPref("username");
	},

	getWikiUrl: function() {
		// add "/" at the end of the url
		if ((wpWikiUrl != null) && (wpWikiUrl != "")) {
			if (wpWikiUrl.charAt(wpWikiUrl.length-1) != "/") {
				wpWikiUrl = wpWikiUrl+"/";
			}
		}
		return wpWikiUrl;
	},

	getWikiLanguage: function() {
		return wpWikiLanguage;
	},

	setLocation: function(name) {
		var wp_homepage = wikipedia.getWikiUrl();
		var url = "";
		switch (name) {
			case 'wikipedia':
				url = wp_homepage+'index.php';
				break;
			case 'toolbar':
				url = 'http://wikipedia.mozdev.org/';
				break;
			case 'user':
				var username = wikipedia.getUsername();
				if (username != "") {
					url = wikipedia.getLink("user_page",new Array(wp_homepage,encodeURIComponent(username)));
				}
				break;
			case 'usertalk':
				username = wikipedia.getUsername();
				if (username != "") {
					url = wikipedia.getLink("user_talk",new Array(wp_homepage,encodeURIComponent(username)));
				}
				break;
			case 'contributions':
				username = wikipedia.getUsername();
				if (username != "") {
					url = wikipedia.getLink("user_contributions",new Array(wp_homepage,encodeURIComponent(username)));
				}
				break;
		}
		if (url != "") {
			wikipedia.setEncodedUrl(url);
		}
		this.addHistory("View Location: " + name, "wikipedia.setLocation('"+name+"')");
	},

	setUrl: function(url) {
		var encodedUrl = encodeURI(url);
		window._content.document.location.href = encodedUrl;
	},

	setEncodedUrl: function(url) {
		window._content.document.location.href = url;
	},

	loadWikiPage: function(title, arguments) {
		var wp_homepage = wikipedia.getWikiUrl();
		var url = wp_homepage+'index.php?title='+encodeURIComponent(title);
		if (arguments != null) {
			for (var i = 0; i < arguments.length; i++) {
				url = url+"&"+encodeURIComponent(arguments[i]);
			}
		}
		wikipedia.setEncodedUrl(url);
	},

	specialPage: function(pageName) {
		wikipedia.loadWikiPage('Special:'+pageName);
		this.addHistory("View Special Page: " + pageName, "wikipedia.specialPage('"+pageName+"')");
	},

	search: function(value) {
		value = trim(value);
		value = replace(value," ","_");
		if (value != "") {
			wikipedia.setEncodedUrl(wikipedia.getLink("wiki_search",new Array(wikipedia.getWikiUrl(),encodeURIComponent(value))));
		}
	},

	searchToolbar: function() {
		wikipedia.search(document.getElementById('wikipedia-search-box').value);
		return true;
	},

	searchSelection: function() {
		if (gContextMenu) {
			var search = trim(gContextMenu.searchSelected());
			if (search == "") {
				search = trim(wikipedia.getSelection());
			}
			wikipedia.search(search);
		}
	},

	toolbarOptions: function() {
		if (wpIsMozilla) {
			goPreferences('navigator.xul', 'chrome://wikipedia/content/wikipedia_preferences_user.xul', 'wikipedia-pref-item');
		} else {
			toOpenWindowByType('global:WikipediaPreferences','chrome://wikipedia/content/wikipedia_preferences_firefox.xul');
		}
	},

	getString:function(name) {
		if (wpStringBundle == null) {
			wpStringBundle = document.getElementById("wikipediastrings");
		}
		return wpStringBundle.getString("wikipedia."+name);
	},

	getInput: function(textId) {
		var dialogText = wikipedia.getString(textId);
		var newtext = prompt(dialogText);
		return newtext;
	},

	getLanguages: function() {
		try {
			var langlist = new Array();
			var nodelist = getNodes("chrome://wikipedia/content/wiki_projects.xml","/languages/language");
			for(var i = 0; i < nodelist.length; i++) {
				node = nodelist[i];
				var code = document.evaluate("code/text()",node,null,0,null);
				code = code.iterateNext().nodeValue;
				var name = document.evaluate("name/text()",node,null,0,null);
				name = name.iterateNext().nodeValue;
				var english = document.evaluate("name_english/text()",node,null,0,null);
				english = english.iterateNext().nodeValue;

				var entry=new Array();
				entry.push(code);
				entry.push(name);
				entry.push(english);
				langlist.push(entry);
			}
			return langlist;
		} 
		catch (e) {
			dump(e);
		}
		return new Array();
	},

	getLanguageString: function(tagName) {
		try {
			var code = "en";
			try {
				code = wikipedia.getPreferences().getCharPref("language");
			}
			catch (e) {
			}
			var nodelist = getNodes("chrome://wikipedia/content/wiki_projects.xml","/languages/language[code='"+code+"']");
			if (nodelist.length == 0) {
				return "";
			}
			var node = nodelist[0];
			var langString = document.evaluate(tagName+"/text()",node,null,0,null);
			langString = langString.iterateNext().nodeValue;
			return langString;
		} 
		catch (e) {
			dump(e);
			return "";
		}
	},

	toggleUserFormat: function(format) {
		var selection = wikipedia.getSelection();
		if (wikipedia.stringHasFormat(selection, format)) {
			wikipedia.removeFormatString(format, 'userformat');
		} else {
			wikipedia.applyFormatString(format);
		}
	},

	showFormatMenu: function(popupMenu) {
		try {
			while (popupMenu.firstChild.getAttribute("id").indexOf("userformat") == 0) {
				popupMenu.removeChild(popupMenu.firstChild);
			}
			var firstChild = popupMenu.firstChild;
			var count = wikipedia.getPreferences().getIntPref(OPTION_USER_FORMAT+".count");
			for (var i = 0; i < count; i++) {
				var label = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_FORMAT+"."+i+".label"));
				var text = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_FORMAT+"."+i+".text"));
				text = replace(text,"SELECTION","%1");
				var menuitem = document.createElement('menuitem');
				menuitem.setAttribute('id', 'userformat'+i);
				menuitem.setAttribute('label', label);
				menuitem.setAttribute('oncommand', 'wikipedia.toggleUserFormat(\''+text+'\')');
				popupMenu.insertBefore(menuitem,firstChild);
			}
			if (count > 0) {
				menuseparator = document.createElement('menuseparator');
				menuseparator.setAttribute('id', 'userformat-separator');
				popupMenu.insertBefore(menuseparator,firstChild);
			}
		}
		catch(e) {
			dump(e)
		}
	},

	getXMLValue: function(file, datalocation, node) {
		var fullfile = "chrome://"+file+"/content/"+file+ "_templates.xml";
		var nodexml = getNodes(fullfile, datalocation);
		nodedata = nodexml[0];
		var value = document.evaluate(node+"/text()",nodedata,null,0,null).iterateNext().nodeValue;
		return value;
	},

	showTemplateSiteMenu: function(popupMenu) {
		var children = popupMenu.childNodes;
		for (var j = children.length-1; j >= 0; j--) {
			if (children[j].getAttribute('id').indexOf('scustom') >= 0) {
				popupMenu.removeChild(children[j]);
			}
		}
		this.loadSiteTemplateSites(popupMenu);
		this.loadSiteTemplateMenuCat(popupMenu);
		this.loadSiteTemplateMenuItems(popupMenu);
	},

	loadSiteTemplateSites: function(popupMenu) {
		//	0 - only show site that we are at.. (defualt)
		//	1 - show only sites with common host, group by lang
		//	2 - show all sites (grouped together)
		//	3 - show all sites (ind.)
		var enabledsites = wikipedia.getPreferences().getIntPref("sitemenugrouping");
		switch (enabledsites) {
		case 1:
			var wcount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI+".count");
			var scount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_SITE+".count");
			for (var i = 0; i < wcount; i++) {
				var installurl = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI+"."+i+".installurl"));
				for (var o = 0; o < scount; o++) {
					var langcode = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_SITE+"."+installurl+"."+o+".code"));
					
					var nvalue = this.getXMLValue(installurl, "/site/"+langcode+"/settings", "fullname");
					var value = this.getXMLValue(installurl, "/site/"+langcode+"/settings", "loc");
					var name = this.getXMLValue(installurl, "/site/overall", "name");

					if (o == 0) { this.addTemplateSiteLabel(popupMenu, "undefined", name, installurl, name); }
					this.addTemplateSiteMenuCat(popupMenu, "site-"+installurl+"-lang-"+langcode, nvalue, "undefined");
				}
			}
			break;
		case 2:
			
			break;
		case 3:
		
			break;
		default:
			var wcount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI+".count");
			var scount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_SITE+".count");
			for (var i = 0; i < wcount; i++) {
				var installurl = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI+"."+i+".installurl"));
				for (var o = 0; o < scount; o++) {
					var langcode = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_SITE+"."+installurl+"."+o+".code"));
					var value = this.getXMLValue(installurl, "/site/"+langcode+"/settings", "loc");
					if (wpCurrentUrl.indexOf(value) >= 0) {
						wpSiteValue = value;
					}
				}
			}	
		}
	},

	loadSiteTemplateMenuCat: function(popupMenu) {
		//	0 - only show site that we are at.. (defualt)
		//	1 - show only sites with common host, group by lang
		//	2 - show all sites (grouped together)
		//	3 - show all sites (ind.)
		var enabledsites = wikipedia.getPreferences().getIntPref("sitemenugrouping");
		switch (enabledsites) {
		case 1:
			var wcount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI+".count");
			var scount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_SITE+".count");
			for (var i = 0; i < wcount; i++) {
				var installurl = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI+"."+i+".installurl"));
				for (var o = 0; o < scount; o++) {
					var langcode = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_SITE+"."+installurl+"."+o+".code"));		
					var nodesitecat = getNodes("chrome://"+installurl+"/content/"+installurl+ "_templates.xml","/site/"+langcode+"/sections/section");
					for (var p = 0; p < nodesitecat.length; p++) {
						nodesitecatdata = nodesitecat[p];
						var label = document.evaluate("label/text()",nodesitecatdata,null,0,null);
						var catid = document.evaluate("area/text()",nodesitecatdata,null,0,null);
						try {
							var parent = document.evaluate("parent/text()",nodesitecatdata,null,0,null).iterateNext().nodeValue;
						}
						catch (e) {
							var parent = "undefined";
						}				
						label = label.iterateNext().nodeValue;
						catid = catid.iterateNext().nodeValue;
						if (parent != "undefined") {
							parentdata = parent;
						} else {
							parentdata = "site-"+installurl+"-lang-"+langcode;
						}
						this.addTemplateSiteMenuCat(popupMenu, catid, label, parentdata);
					}
				}
			}
			break;
		case 2:
			
			break;
		case 3:
			
			break;	
		default:

		}
	},

	loadSiteTemplateMenuItems: function(popupMenu) {
		//	0 - only show site that we are at.. (defualt)
		//	1 - show only sites with common host, group by lang
		//	2 - show all sites (grouped together)
		//	3 - show all sites (ind.)
		var enabledsites = wikipedia.getPreferences().getIntPref("sitemenugrouping");
		switch (enabledsites) {
		case 1:
			var wcount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI+".count");
			var scount = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_SITE+".count");
			for (var i = 0; i < wcount; i++) {
				var installurl = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI+"."+i+".installurl"));
				for (var o = 0; o < scount; o++) {
					var langcode = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_SITE+"."+installurl+"."+o+".code"));		
					var nodesiteitems = getNodes("chrome://"+installurl+"/content/"+installurl+ "_templates.xml","/site/"+langcode+"/templates/template");
					for (var p = 0; p < nodesiteitems.length; p++) {
						nodesiteitemsdata = nodesiteitems[p];
						
						var itemid = document.evaluate("id/text()",nodesiteitemsdata,null,0,null);
						var label = document.evaluate("label/text()",nodesiteitemsdata,null,0,null);
						var catid = document.evaluate("cat/text()",nodesiteitemsdata,null,0,null);
						try {
							var tooltip = document.evaluate("tooltip/text()",nodesiteitemsdata,null,0,null).iterateNext().nodeValue;
						}
						catch (e) {
							var tooltip = null;
						}
						itemid = itemid.iterateNext().nodeValue;
						label = label.iterateNext().nodeValue;
						catid = catid.iterateNext().nodeValue;
						this.addTemplateSiteMenuItem(popupMenu, itemid, langcode, catid, label, installurl, tooltip);
					}
				}
			}
			break;
		case 2:
			
			break;
		case 3:
			
			break;	
		default:

		}
	},

	addTemplateSiteMenuCat: function(popupMenu, catid, label, parent) {
		var item = document.createElement('menu');
		var menupopup = document.createElement('menupopup');		
		menupopup.setAttribute("id", 'context-wikipedia-template-scustom-cat-menupopup-' + catid);
		item.setAttribute("id", 'context-wikipedia-template-scustom-cat-' + catid);
		item.setAttribute('label', label);
		item.setAttribute('class', 'menuitem-iconic menu-iconic');
		item.appendChild(menupopup);
		if (parent != "undefined") {
			document.getElementById('context-wikipedia-template-scustom-cat-menupopup-' + parent).appendChild(item);
		} else {
			popupMenu.appendChild(item);
		}
	},
	
	addTemplateSiteMenuItem: function(popupMenu, itemid, langid, catid, label, installurl, tooltip) {
		var menuitem = document.createElement('menuitem');
		menuitem.setAttribute('id', 'context-wikipedia-template-scustom-cat-menupopup-menuitem-' + itemid);
		menuitem.setAttribute('label', label);
		menuitem.setAttribute('oncommand', 'wikipedia.stmplinsert(\''+label+'\',\''+itemid+'\',\''+installurl+'\',\''+langid+'\')');	//	add installurl and langid
		if (tooltip) {
			menuitem.setAttribute('tooltip', tooltip);
		} else {
			menuitem.setAttribute('tooltip', '');
		}
		if (catid != "undefined") {
			document.getElementById('context-wikipedia-template-scustom-cat-menupopup-' + catid).appendChild(menuitem);
		} else {
			popupMenu.appendChild(menuitem);
		}
	},
	
	addTemplateSiteLabel: function(popupMenu, id, label, installurl, tooltip) {
		var menuitem = document.createElement('menuitem');
		menuitem.setAttribute('id', 'context-wikipedia-template-scustom-cat-menupopup-menuitem-' + installurl);
		menuitem.setAttribute('label', label);
		menuitem.setAttribute('class', 'menuitem-iconic menu-iconic menu-label');
		//menuitem.setAttribute('disabled', "true");
		if (tooltip) {
			menuitem.setAttribute('tooltip', tooltip);
		} else {
			menuitem.setAttribute('tooltip', '');
		}
		if (id != "undefined") {
			document.getElementById('context-wikipedia-template-scustom-cat-menupopup-' + id).appendChild(menuitem);
		} else {
			popupMenu.appendChild(menuitem);
		}
	},
	
	stmplinsert: function(label, tmplcode, installurl, langcode) {
		var nodesiteitems = getNodes("chrome://"+installurl+"/content/"+installurl+ "_templates.xml","/site/"+langcode+"/templates/template");
		for (var p = 0; p < nodesiteitems.length; p++) {
			nodesiteitemsdata = nodesiteitems[p];
			var itemid = document.evaluate("id/text()",nodesiteitemsdata,null,0,null);
			var text = document.evaluate("text/text()",nodesiteitemsdata,null,0,null);
			itemid = itemid.iterateNext().nodeValue;
			text = text.iterateNext().nodeValue;
			if (itemid == tmplcode) {
				//	TODO - Version v0.8.2 (Process Optionial Data)
				this.insert(text);
				this.addHistory("Site Template: " + label, "wikipedia.stmplinsert('"+label+"','"+tmplcode,+"','"+installurl+"','"+langcode+"')");				
			}
		}
	},

	showTemplateMenu: function(popupMenu) {
		var children = popupMenu.childNodes;
		for (var j = children.length-1; j >= 0; j--) {
			if (children[j].getAttribute('id').indexOf('ucustom') >= 0) {
				popupMenu.removeChild(children[j]);
			}
		}
		this.loadTemplateMenuCat(popupMenu);
		this.loadTemplateMenuItems(popupMenu);
	},
	
	loadTemplateMenuCat: function(popupMenu) {
		var count = wikipedia.getPreferences().getIntPref(OPTION_USER_TEMPLATE_CAT+".count");
		for (var i = 0; i < count; i++) {
			var catid = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE_CAT+"."+i+".cat"));
			var label = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE_CAT+"."+i+".label"));
			var parent = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE_CAT+"."+i+".parent"));
			this.addTemplateMenuCat(popupMenu, catid, label, parent);
		}
	},
	
	loadTemplateMenuItems: function(popupMenu) {
		var count = wikipedia.getPreferences().getIntPref(OPTION_USER_TEMPLATE+".count");
		for (var i = 0; i < count; i++) {
			var catid = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE+"."+i+".cat"));
			var label = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE+"."+i+".label"));
			var templatecode = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE+"."+i+".templatecode"));
			var tooltip = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE+"."+i+".tooltip"));
			this.addTemplateMenuItem(popupMenu, catid, label, templatecode, tooltip);
		}
	},

	addTemplateMenuCat: function(popupMenu, catid, label, parent) {
		var item = document.createElement('menu');
		var menupopup = document.createElement('menupopup');
		item.setAttribute("id", 'context-wikipedia-template-ucustom-cat-' + catid);
		item.setAttribute('label', label);
		item.setAttribute('class', 'menuitem-iconic menu-iconic');
		menupopup.setAttribute("id", 'context-wikipedia-template-ucustom-cat-menupopup-' + catid);
		item.appendChild(menupopup);
		if (parent != "undefined") {
			document.getElementById('context-wikipedia-template-ucustom-cat-menupopup-' + parent).appendChild(item);
		} else {
			popupMenu.appendChild(item);
		}
	},
	
	addTemplateMenuItem: function(popupMenu, catid, label, templatecode, tooltip) {
		var menuitem = document.createElement('menuitem');
		menuitem.setAttribute('id', 'context-wikipedia-template-ucustom-cat-menupopup-menuitem-' + label);
		menuitem.setAttribute('label', label);
		menuitem.setAttribute('oncommand', 'wikipedia.tmplinsert(\''+label+'\',\''+templatecode+'\')');
		if (tooltip) {
			menuitem.setAttribute('tooltip', tooltip);
		} else {
			menuitem.setAttribute('tooltip', '');
		}
		if (catid != "undefined") {
			document.getElementById('context-wikipedia-template-ucustom-cat-menupopup-' + catid).appendChild(menuitem);
		} else {
			popupMenu.appendChild(menuitem);
		}
	},

	tmplinsert: function(label, tmplcode) {
		var count = wikipedia.getPreferences().getIntPref(OPTION_USER_TEMPLATE+".count");
		for (var i = 0; i < count; i++) {
			var templatecode = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE+"."+i+".templatecode"));
			var text = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_TEMPLATE+"."+i+".text"));
			//	TODO - Version v0.8.2 (Process Optionial Data)
			if (templatecode == tmplcode) {
				this.insert(text);
				this.addHistory("User Template: " + label, "wikipedia.tmplinsert('"+label+"','"+templatecode+"')");
			}
		}
	},
	
	showHistory: function(popupMenu) {
		var children = popupMenu.childNodes;
		for (var j = children.length-1; j >= 0; j--) {
			if (children[j].getAttribute('id').indexOf('menu') >= 0) {
				popupMenu.removeChild(children[j]);
			}
		}
		this.loadHistory(popupMenu);
	},

	loadHistory: function(popupMenu) {
		var count = wikipedia.getPreferences().getIntPref(OPTION_HISTORY+".count");
		for (var i = 0; i < count; i++) {
			var label = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_HISTORY+"."+i+".label"));
			var command = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_HISTORY+"."+i+".item"));
			this.addHistoryMenuItem(popupMenu, label, command);
		}
	},

	addHistoryMenuItem: function(popupMenu, label, command) {
		var menuitem = document.createElement('menuitem');
		menuitem.setAttribute('id', 'context-wikipedia-history-menu-menuitem-' + label);
		menuitem.setAttribute('label', label);
		menuitem.setAttribute('oncommand', command);
		popupMenu.appendChild(menuitem);
	},

	addHistory: function(label, command) {
		var count = wikipedia.getPreferences().getIntPref(OPTION_HISTORY+".count");	//	current number of history commands
		var maxcount = wikipedia.getPreferences().getIntPref(OPTION_HISTORY+".max.count");	// total amount to save
		if (count < maxcount) {
			//	add a command in the history, do not replace any yet because
			//	we are still under the max
			wikipedia.getPreferences().setCharPref(OPTION_HISTORY+"."+count+".label", encodeURIComponent(label));				
			wikipedia.getPreferences().setCharPref(OPTION_HISTORY+"."+count+".item", encodeURIComponent(command));
			count = count+1;
			wikipedia.getPreferences().setIntPref(OPTION_HISTORY+".count", count);
		} else {
			//	we have reached the max, replace the oldest one first, move the rest down
			//	1 (oldest) maxcount (newest)
			
		}
	},

	showQuotationMenu: function(popupMenu) {
		try {
			if (popupMenu.childNodes.length == 0) {
				var nodelist = getNodes("chrome://wikipedia/content/quotation_marks.xml","/quotation_marks/quotation[code='"+wikipedia.getWikiLanguage()+"']");
				for(var i = 0; i < nodelist.length; i++) {
					var node = nodelist[i];
					wikipedia.addQuote("standard_double", node, popupMenu);
					wikipedia.addQuote("standard_single", node, popupMenu);
					wikipedia.addQuote("alternative_double", node, popupMenu);
					wikipedia.addQuote("alternative_single", node, popupMenu);
					popupMenu.appendChild(document.createElement('menuseparator'));
				}
				nodelist = getNodes("chrome://wikipedia/content/quotation_marks.xml","/quotation_marks/quotation");
				for(var i = 0; i < nodelist.length; i++) {
					node = nodelist[i];
					var language = document.evaluate("language/text()",node,null,0,null);
					language = language.iterateNext().nodeValue;
					var quoteLangMenu = document.createElement('menu');
					quoteLangMenu.setAttribute('label', language);
					popupMenu.appendChild(quoteLangMenu);
					var quoteLangPopup = document.createElement('menupopup');
					quoteLangMenu.appendChild(quoteLangPopup);
					wikipedia.addQuote("standard_double", node, quoteLangPopup);
					wikipedia.addQuote("standard_single", node, quoteLangPopup);
					wikipedia.addQuote("alternative_double", node, quoteLangPopup);
					wikipedia.addQuote("alternative_single", node, quoteLangPopup);
				}
			}
		} 
		catch (e) {
			dump(e);
		}
	},

	addQuote: function(quoteName, node, quoteLangPopup) {
		var quote = document.evaluate(quoteName+"/text()",node,null,0,null);
		quote = quote.iterateNext();
		if (quote != null) {
			quote = quote.nodeValue;
			var left = quote.substring(0,(quote.length-1)/2);
			var right = quote.substring((quote.length+1)/2,quote.length);
			var quoteItem = document.createElement('menuitem');
			quoteItem.setAttribute('label', quote+" - "+wikipedia.getString("quotation."+quoteName));
			quoteItem.setAttribute('oncommand', 'wikipedia.surround(\'quotation\',\''+left+'\',\''+right+'\')');
			quoteLangPopup.appendChild(quoteItem);
		}
	},

	showProjectMenu: function(menuid) {
		var children = menuid.childNodes;
		for (var j = children.length-1; j >= 0; j--) {
			menuid.removeChild(children[j]);
		}
		//	add standard wikis
		var count = wikipedia.getPreferences().getIntPref(OPTION_ACTIVE+".count");
		for (var i = 0; i < count; i++) {
			var url = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+"."+i+".url");
			var label = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+"."+i+".label");
			var language = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+"."+i+".language");
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", label);
			menuitem.setAttribute("oncommand", "wikipedia.selectProject(\""+label+"\",\""+url+"\",\""+language+"\",\"mediawiki\");");
			menuid.appendChild(menuitem);
		}
		
		//	user wikiks - installed wikis files
		count = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI+".count");
		for (var i = 0; i < count; i++) {	
			var installurl = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI+"."+i+".installurl");
			var fullfile = "chrome://"+installurl+"/content/"+installurl+"_templates.xml";
			var nodelistsite = getNodes(fullfile,"/site/overall");
			nodesite = nodelistsite[i];

			var name = document.evaluate("name/text()",nodesite,null,0,null);
			var type = document.evaluate("type/text()",nodesite,null,0,null);
			var icon = document.evaluate("icon/text()",nodesite,null,0,null);
			var thelang = document.evaluate("lang/text()",nodesite,null,0,null);
			
			name = name.iterateNext().nodeValue;
			type = type.iterateNext().nodeValue;
			icon = icon.iterateNext().nodeValue;
			
			var thelang = thelang.iterateNext().nodeValue;
			var lang = thelang.split(",");
					
			for (var l = 0; l < lang.length; l++) {
				var nodelistlang = getNodes(fullfile,"/site/"+lang[l]+"/settings");
				nodelang = nodelistlang[0];
				var loc = document.evaluate("loc/text()",nodelang,null,0,null);
				loc = loc.iterateNext().nodeValue;
				var count_site_active = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_SITE+".count");
				for (var s = 0; s < count_site_active; s++) {
					var active_code = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_SITE+"."+installurl+"."+s+".code");
					if (active_code == lang[l]) {
						var menuitem = document.createElement("menuitem");
						menuitem.setAttribute("label", name+" ("+active_code+")");
						menuitem.setAttribute("oncommand", "wikipedia.selectProject(\""+name+" ("+active_code+")"+"\",\""+loc+"\",\""+active_code+"\",\""+type+"\");");
						menuid.appendChild(menuitem);
					}
				}
			}
		}
		
		//	user wiki basic
		var count = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_NOINSTALL+".count");
		for (var i = 0; i < count; i++) {
			var url = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_NOINSTALL+"."+i+".url"));
			var label = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_NOINSTALL+"."+i+".label"));
			var language = decodeURIComponent(wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_NOINSTALL+"."+i+".language"));
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", label + " ("+language+")");
			menuitem.setAttribute("oncommand", "wikipedia.selectProject(\""+label+"\",\""+url+"\",\""+language+"\",\"mediawiki\");");
			menuid.appendChild(menuitem);
		}
		
		return true;
	},

	selectProject: function(label,url,language,type) {
		var currentUrl = window._content.document.location.href;
		document.getElementById("project-name").setAttribute("value",label);
		var loadHomepage = (wikipedia.getPreferences().getBoolPref("homeonchange") && (wpWikiUrl != ''));
		wpWikiUrl = url;
		wpWikiLanguage = language;
		wpWikiType = type;
		if ((loadHomepage) && (wpCurrentUrl.indexOf(url) < 0)) {
			wikipedia.setLocation('wikipedia');
		}
		wikipedia.getPreferences().setCharPref(OPTION_ACTIVE+".label",label);
		wikipedia.getPreferences().setCharPref(OPTION_ACTIVE+".url",url);
		wikipedia.getPreferences().setCharPref(OPTION_ACTIVE+".language",language);
		wikipedia.getPreferences().setCharPref(OPTION_ACTIVE+".type",type);
		wikipedia.switchWikiType();
	},

	setCurrentProject: function() {
		var label = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+".label");
		var url = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+".url");
		var language = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+".language");
		var type = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+".type");
		wikipedia.selectProject(label,url,language,type);
	},

	getArticleName: function() {
		var url = window._content.document.location.href;
		if (document.popupNode.tagName == "A") {
			url = document.popupNode.toString();
		}
		if (url != "") {
			url = trim(url);
			if (url == "") {
				return "";
			}
			var mainUrl = wikipedia.getWikiUrl();
			// try finding by removing wiki url
			if (url.indexOf(mainUrl) == 0) {
				var article = url.substring(mainUrl.length);
				return decodeURIComponent(article);
			}
			// fallback: try everything after last slash
			var pos = url.lastIndexOf("/");
			if ((pos > 0) && (pos < url.length-1)) {
				return decodeURIComponent(url.substring(pos+1,url.length));
			}
		}
		return "";
	},

	articlePage: function(pageId) {
		var articleName = wikipedia.getArticleName();
		if (articleName != "") {
			var url = wikipedia.getLink(pageId,new Array(wikipedia.getWikiUrl(),encodeURIComponent(articleName)));
			if (url != "") {
				wikipedia.setEncodedUrl(url);
			}
		}
		return true;
	},

	pageAddUserWiki: function() {
		if (document.popupNode.tagName == "A") {
			var link = document.popupNode.toString();
		} else {
			link = wpCurrentUrl;
		}
		if (link != "") {
			var ok = [false];
			var params = ["","","basic",link];
			window.openDialog("chrome://wikipedia/content/wikipedia_wiki_properties.xul", "&wikipedia.wiki.dialogtitle;","chrome,modal,centerscreen,resizable",ok,params);
		}
		return true;
	},

	getPreferences: function() {
		if (!wpPrefBranch) {
			try {
				var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
				wpPrefBranch = prefService.getBranch("wikipedia."); // preferences wikipedia node
				wpPrefBranch.setIntPref("lastversion",70);
			} 
			catch (e) {
				alert(e);
			}
		}
		return wpPrefBranch;
	},

	checkAllBoxes: function() {
		var htmlDoc = window._content.document;
		if (htmlDoc != null) {
			var checkBoxes = htmlDoc.getElementsByTagName("input");
			for (var i = 0; i < checkBoxes.length; i++) {
				var checkBox = checkBoxes[i];
				if (checkBox.getAttribute("type") == "checkbox") {
					checkBox.setAttribute("checked","checked");
				}
			}
		}
	},

	checkBoxesAvailable: function() {
		try {
			var htmlDoc = window._content.document;
			if (htmlDoc != null) {
				var checkBoxes = htmlDoc.getElementsByTagName("input");
				for (var i = 0; i < checkBoxes.length; i++) {
					var checkBox = checkBoxes[i];
					if (checkBox.getAttribute("type") == "checkbox") {
						return true;
					}
				}
			}
		}
		catch (e) {
		}
		return false;
	},

	switchWikiType: function() {
		// read format strings into a cache to not lookup each format
		var nodeList = getNodes("chrome://wikipedia/content/wikisoftware/"+wpWikiType+".xml","/wikisoftware/formats/format");
		var wpSupportedFormats = new Array();
		for (var i = 0; i < nodeList.length; i++) {
			var node = nodeList[i];
			var formatId = node.getAttribute("id");
			var format = node.getAttribute("formatstring");
			if (format != "") {
				wpSupportedFormats.push(formatId);
			}
		}
		// now adapt GUI to the possibilities of the wiki implementation
		for (var j = 0; j < wpAllFormats.length; j++) {
			formatId = wpAllFormats[j];
			var supported = false;
			for (var i = 0; i < wpSupportedFormats.length; i++) {
				if (wpSupportedFormats[i] == formatId) {
					supported = true;
				}
			}
			var element = document.getElementById("context-wikipedia-"+formatId);
			if (element != null) {
				element.hidden = !supported;
			}
		}
	},

	getLink: function(linkId, arguments) {
		var nodeList = getNodes("chrome://wikipedia/content/wikisoftware/"+wpWikiType+".xml","/wikisoftware/links/link[attribute::id='"+linkId+"']");
		if (nodeList.length > 0) {
			var node = nodeList[0];
			var link = node.getAttribute("formatstring");
			for (var i = 0; i < arguments.length; i++) {
				var regEx = new RegExp ('%'+(i+1), 'gi') ;
				link = link.replace(regEx, arguments[i]);
			}
			return link;
		}
		return "";
	},

	articleInfo: function() {
        var ok = [false];
        var str = currentBox.value;
        var params = [str];
        window.openDialog("chrome://wikipedia/content/wikipedia_article_info.xul", "WikiInfo","chrome,centerscreen,resizable",ok,params);
	},

	openSpecialCharactersDialog: function() {
		var ok = [false];
		var params = ["",wikipedia];
		window.openDialog("chrome://wikipedia/content/wikipedia_special.xul", "WikiSpecialChars","chrome,close,centerscreen,resizable",ok,params);
	},

	isEditFieldSelected: function() {
		var focused = document.commandDispatcher.focusedElement;
		if (focused != null) {
			return (focused.tagName == "TEXTAREA");
		}
		return false;
	},

	isWritableEditFieldSelected: function() {
		var focused = document.commandDispatcher.focusedElement;
		if (focused != null) {
			var enable = (focused.tagName == "TEXTAREA");
			if (enable) {
				return (focused.getAttribute("readonly") != "readonly");
			}
		}
		return false;
	},

	//observe: method of the nsIObserver interface, called on preference changes
	observe: function(subject,topic,data) {
		if (topic == "nsPref:changed") {
			dataString = "" + data;
			if ((dataString == "wikipedia.activation.kind") || (dataString == "wikipedia.activation.condition")) {
				wikipedia.showHideToolbar();
			}
		}
	},

	onStateChange: function(aProgress,aRequest,aFlag,aStatus) {
		if (aFlag == 786448) {
			wikipedia.focusEditField();
		}
	},

	onLocationChange:function(a,b,c) {
		if (c != null) {
			if (c.spec != wpCurrentUrl) {
				wpIsEditPage = false;
				wpCurrentUrl = c.spec;
				wikipedia.showHideToolbar();
				wikipedia.checkForWikiUrl();
				wikipedia.focusEditField();
			}
		}
	},

	onProgressChange: function(a,b,c,d,e,f) {
	},
	onStatusChange: function(a,b,c,d) {
	},
	onSecurityChange: function(a,b,c) {
	},
	
/*
	This is not nsIWebProgressListenr method,
	just killing a error in tabbrowser.xml
	Maybe a bug.
*/
	onLinkIconAvailable:function(a) {
	},

	checkForWikiUrl: function() {
		var count = wikipedia.getPreferences().getIntPref(OPTION_ACTIVE+".count");
		for (var i = 0; i < count; i++) {
			var url = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+"."+i+".url");
			if (wpCurrentUrl.indexOf(url) == 0) {
				var label = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+"."+i+".label");
				var language = wikipedia.getPreferences().getCharPref(OPTION_ACTIVE+"."+i+".language");
				wikipedia.selectProject(label,url,language, "mediawiki");
				return;
			}
		}
		
		//	user wikiks - installed wikis files
		count = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI+".count");
		for (var i = 0; i < count; i++) {	
			var installurl = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI+"."+i+".installurl");
			var fullfile = "chrome://"+installurl+"/content/"+installurl+"_templates.xml";
			var nodelistsite = getNodes(fullfile,"/site/overall");
			nodesite = nodelistsite[i];

			var name = document.evaluate("name/text()",nodesite,null,0,null);
			var type = document.evaluate("type/text()",nodesite,null,0,null);
			var icon = document.evaluate("icon/text()",nodesite,null,0,null);
			var thelang = document.evaluate("lang/text()",nodesite,null,0,null);
			
			name = name.iterateNext().nodeValue;
			type = type.iterateNext().nodeValue;
			icon = icon.iterateNext().nodeValue;
			
			var thelang = thelang.iterateNext().nodeValue;
			var lang = thelang.split(",");
						
			for (var l = 0; l < lang.length; l++) {
				var nodelistlang = getNodes(fullfile,"/site/"+lang[l]+"/settings");
				nodelang = nodelistlang[0];
				var loc = document.evaluate("loc/text()",nodelang,null,0,null);
				loc = loc.iterateNext().nodeValue;
				var count_site_active = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_SITE+".count");
				for (var s = 0; s < count_site_active; s++) {
					var active_code = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_SITE+"."+installurl+"."+s+".code");
					if (active_code == lang[l]) {
						if (wpCurrentUrl.indexOf(loc) == 0) {
							wikipedia.selectProject(name+" ("+active_code+")",loc,active_code,type);
							return;
						}
					}
				}
			}
		}

		//	basic install pack
		var count = wikipedia.getPreferences().getIntPref(OPTION_USER_WIKI_NOINSTALL+".count");
		for (var i = 0; i < count; i++) {
			var url = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_NOINSTALL+"."+i+".url");
			if (wpCurrentUrl.indexOf(url) == 0) {
				var label = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_NOINSTALL+"."+i+".label");
				var language = wikipedia.getPreferences().getCharPref(OPTION_USER_WIKI_NOINSTALL+"."+i+".language");
				wikipedia.selectProject(label+" ("+language+")",url,language, "mediawiki");
				return;
			}
		}
	},

	focusEditField: function() {
		if (wpCurrentUrl.indexOf('action=edit') > 0) {
			var htmlDoc = window._content.document;
			if (htmlDoc != null) {
				if (wpDocument.getElementById('wikipedia-'+wpEnableButtonIds[0]).disabled) {
					var textareas = htmlDoc.getElementsByTagName('textarea');
					for (var i = 0; i < textareas.length; i++) {
						var textarea = textareas[i];
						if (textarea.getAttribute('name') == 'wpTextbox1') {
							wpIsEditPage = true;
/*							textarea.focus();
							wikipedia.focusEvent();
							textarea.selectionStart=0;
							textarea.selectionEnd=0;
*/							return;
						}
					}
				}
			}
		}
	}
};

//load wikipedia on startup
window.addEventListener('load', wikipedia.init, false);