/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distribselectGameuted on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Wikipedia Toolbar.
 *
 * The Initial Developer of the Original Code is
 * Bananeweizen
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Shane A. Froebel <shane.froebel@gmail.com/bugs@bugssite.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 /* ***** CVS HEADERS *****
 * $Id: wikipedia_article_info.js,v 1.3 2006/04/11 14:27:34 froebel Exp $
 * ***** END CVS HEADERS ***** */
 
var countProblems = 0;
var linkCounts = null;
var wpStringBundle = null;

function onLoad() {
	wpStringBundle = document.getElementById("wikipediastrings");
	var params = window.arguments[1];
	var text = params[0];
	listTemplates(text);
	listLinks(text);
	checkProblems(text);
	articleStats(text);
}

function onAccept() {
	return true;
}

function listItems(text, parentId, open, close) {
	var linkTree = document.getElementById(parentId);
	linkCounts = new Array();
	var start = text.indexOf(open);
	var end = text.indexOf(close,start);
	while ((start >= 0) && (end >= 0)) {
		var end = text.indexOf(close,start);
		if (end > 0) {
			var link = text.substring(start+2,end);
			if (link.indexOf("|") > 0) {
				link = link.substring(0,link.indexOf("|"));
			}
			link = trim(link);
			var count = 1;
			var found = false;
			for (var i = 0; i < linkCounts.length; i++) {
				var linkCount = linkCounts[i];
				if (linkCount[0] == link) {
					count = linkCount[1]+1;
					linkCounts[i] = new Array(link,count);
					found = true;
					break;
				}
			}
			if (!found) {
				linkCount = new Array(link,count);
				linkCounts.push(linkCount);
			}
		}
		// search next
		var start = text.indexOf(open, end);
	}
	//output links
	linkCounts.sort();
	var count = 0;
	for (var i = 0; i < linkCounts.length; i++) {
		linkCount = linkCounts[i];
		var item = document.createElement('treeitem');
		var row = document.createElement('treerow');
		var cell = document.createElement('treecell');
		cell.setAttribute("label",linkCount[0]);
		row.appendChild(cell);
		cell= document.createElement('treecell');
		cell.setAttribute("label",linkCount[1]);
		count= count+linkCount[1];
		row.appendChild(cell);
		item.appendChild(row);
		linkTree.appendChild(item);
	}
	return count;
}

function listLinks(text) {
	document.getElementById("statistics-links").setAttribute("value",listItems(text, "items-links", "[[", "]]"));
}

function listTemplates(text) {
	document.getElementById("statistics-templates").setAttribute("value",listItems(text, "items-templates", "{{", "}}"));
}

function countWords(text) {
	var start = text.indexOf("<");
	while (start >= 0) {
		var end = text.indexOf(">",start);
		text = text.substring(0,start-1)+text.substring(end+1,text.length);
		start = text.indexOf("<");
	}
//	alert(text);
	var count = 0;
	var lastAlNum = false;
	for (var i = 0; i < text.length; i++) {
		var c = text.charAt(i);
		var alNum = (('a' <= c) && (c <= 'z'))
			||(('A' <= c) && (c <= 'Z'))
			||(('0' <= c) && (c <= '9'))
			||('äöüß'.indexOf(c) >= 0);
		if (alNum && !lastAlNum) {
			count++;
		}
		lastAlNum = alNum;
	}
	document.getElementById("statistics-words").setAttribute("value","~"+count);
}

function articleStats(text) {
	document.getElementById("statistics-bytes").setAttribute("value",text.length);
	countWords(text);
}

function numberOf(text,sub) {
	var count = 0;
	var pos = text.indexOf(sub);
	while (pos >= 0) {
		count++;
		pos = text.indexOf(sub,pos+1);
	}
	return count;
}

function addProblem(text) {
	var problems = document.getElementById("items-problems");
	var item = document.createElement('treeitem');
	var row = document.createElement('treerow');
	var cell = document.createElement('treecell');
	cell.setAttribute("label",text);
	row.appendChild(cell);
	item.appendChild(row);
	problems.appendChild(item);
	countProblems++;
}

function checkEqual(text,open,close) {
	if (numberOf(text,open) != numberOf(text,close)) {
		addProblem(formatString(wpStringBundle.getString("wikipedia.checks.openclose"),new Array(open,close)));
	}
}

function checkHeaders(text) {
	var parts = text.split("\n");
	for (var i = 0; i < parts.length; i++) {
		var part = trim(parts[i]);
		if (part.indexOf("=") == 0) {
			var start = 0;
			while (part.charAt(start) == '=') {
				start++;
			}
			var end = 0;
			while (part.charAt(part.length-end-1) == '=') {
				end++;
			}
			if (start != end) {
				addProblem(formatString(wpStringBundle.getString("wikipedia.checks.headerformat"),new Array(part)));
			}
		}
	}
}

function checkProblems(text) {
	var countProblems = 0;
	var wbPrefs = getPreferences();
	if (wbPrefs.getBoolPref("checks.openclose")) {
		checkEqual(text,"[","]");
		checkEqual(text,"{","}");
		checkEqual(text,"(",")");
	}
	if (wbPrefs.getBoolPref("checks.headers")) {
		checkHeaders(text);
	}
	if (wbPrefs.getBoolPref("checks.multiplelink")) {
		for (var i = 0; i < linkCounts.length; i++) {
			var linkCount = linkCounts[i];
			if (linkCount[1] > 1) {
				addProblem(formatString(wpStringBundle.getString("wikipedia.checks.multiplelinks"),new Array(linkCount[0])));
			}
		}
	}
	if (countProblems == 0) {
		addProblem(wpStringBundle.getString("wikipedia.checks.noproblems"));
	}
}